#!/usr/bin/env python

"""
Variable-length integer functions.

Copyright (C) 2009 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from array import array

vint_cache = {}
vint_bytes_cache = {}

def vint(number):

    "Write 'number' as a variable-length integer."

    try:
        return vint_cache[number]
    except KeyError:
        if number >= 0:
            bytes = array('B')
            _vint_to_array(number, bytes)
            return bytes.tostring()

        # Negative numbers are not supported.

        else:
            raise ValueError, "Number %r is negative." % number

def vint_to_array(number, bytes):

    "Write 'number' as a variable-length integer to 'bytes'."

    try:
        bytes += vint_bytes_cache[number]
    except KeyError:
        if number >= 0:
            _vint_to_array(number, bytes)

        # Negative numbers are not supported.

        else:
            raise ValueError, "Number %r is negative." % number

def _vint_to_array(number, bytes):

    "Write the 'number' to 'bytes' from least to most significant digits."

    while number > 127:
        bytes.append(number & 127 | 128)
        number = number >> 7
    else:
        bytes.append(number)

for i in xrange(0, 65536):
    bytes = array('B')
    _vint_to_array(i, bytes)
    vint_bytes_cache[i] = bytes
    vint_cache[i] = bytes.tostring()

# vim: tabstop=4 expandtab shiftwidth=4
